
#include "YourPrimaryGeneratorAction.hh"

#include "YourDetectorConstruction.hh"
#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"

YourPrimaryGeneratorAction::YourPrimaryGeneratorAction(YourDetectorConstruction* det)
:  G4VUserPrimaryGeneratorAction(),
   fDetector(det),
   fGun(nullptr) {
   // construct our particle gun that will be used to generate primaries
   G4int numParticles = 1;
   fGun = new G4ParticleGun(numParticles);
   SetDefault(); 
}

YourPrimaryGeneratorAction::~YourPrimaryGeneratorAction() {
   // delete the gun object that was costructed in the constructor 
   delete fGun;
}


void YourPrimaryGeneratorAction::GeneratePrimaries(G4Event* evt) {
   // NOTE: updating the gun position is required since the target thickness 
   //       might have been changed between the invocation of the constructor
   //       of YourDetectorConstruction and YourPrimaryGeneratorAction. While 
   //       updating the postion would be sufficient only once, at the very 
   //       beginning of each run, before the generation of the first event,
   //       we have no possibility to do that at the moment since we did not 
   //       implement any further actions. 
   //       Therefore, this is an inappropriate, only temporal solution (that 
   //       can be very inefficient since it updates the gun position before 
   //       each event). When we will have our G4UserRunAction (YourRunAction) 
   //       implemented, we will move to the appropriate solution: the gun  
   //       position will be updated in the G4UserRunAction::BeginOfRunAction() 
   //       interface method implementation, since that is called by the kernel 
   //       at the beginning of each run BEFORE the FIRST EVENT starts.    
   UpdatePosition();
   // 
   // use our particle gun with its settings to generate the event
   fGun->GeneratePrimaryVertex(evt);
}


void YourPrimaryGeneratorAction::SetDefault() {
   // e-  // G4Electron::Definition()
   G4ParticleDefinition* part = G4ParticleTable::GetParticleTable()->FindParticle("e-");
   // you might chek if ( part == nullptr) !!!
   fGun->SetParticleDefinition(part);
   fGun->SetParticleEnergy(30.0*CLHEP::MeV);
   fGun->SetParticleMomentumDirection( G4ThreeVector(1.0, 0.0, 0.0) );
   UpdatePosition();
}


void YourPrimaryGeneratorAction::UpdatePosition() {
   // ask the detector to provide the appropriate x-position of the gun 
   fGun->SetParticlePosition( G4ThreeVector( fDetector->GetGunXPosition(), 0.0, 0.0  ) );
}

